import json
import re
import capstone
import pefile
from malduck import rc4
from malduck.extractor import Extractor


class NetWire(Extractor):
    family = "netwire"
    yara_rules = (
        "netwire_config0",
        "netwire_config1",
        "netwire_config2",
        "netwire_config3",
    )

    @Extractor.extractor("key")
    def netwire_config(self, p, addr):
        if p.memory:
            decrypt_memory(addr, p.memory)


def get_pe_data(pe, va, length=None):
    image_base = pe.OPTIONAL_HEADER.ImageBase
    rva = va - image_base
    data = pe.get_data(rva, length=length)
    return data


def yara_scan(data, rule_name, compiled_rule):
    matches = compiled_rule.match(data=data)
    output = matches[0].strings[0][0]
    return output


def parse_config(config):
    netwire_config = dict()
    netwire_config["Family"] = "NetWire"
    domain_list = config[0].rstrip(";").split(";")
    netwire_config["C2 IP list"] = domain_list
    netwire_config["Password"] = config[2]
    netwire_config["Host ID"] = config[3]
    netwire_config["Mutex"] = config[4]
    netwire_config["Installation path"] = config[5]
    netwire_config["Run registry key entry"] = config[6]
    netwire_config["Active setup Key"] = config[7]
    netwire_config["Keylogger logs directory"] = config[8]
    netwire_config["Sleep in seconds"] = int(config[11])
    return netwire_config


def decrypt_memory(yara_offset, data):
    print("[+] Rule matched")
    pe = pefile.PE(data=data)
    config = list()
    disassembler = capstone.Cs(capstone.CS_ARCH_X86, capstone.CS_MODE_32)
    disassembler.detail = True

    extracted_values_list = list()
    instructions_bytes = get_pe_data(pe, yara_offset)
    for _, instruction in enumerate(disassembler.disasm(instructions_bytes, 0)):
        if (
            (
                len(instruction.operands) > 1
                and instruction.operands[1].type == capstone.CS_OP_IMM
            )
            and instruction.mnemonic == "mov"
            and instruction.operands[0].type == capstone.CS_OP_MEM
        ):
            extracted_values_list.append(instruction.operands[1].value.imm)

        if (
            instruction.mnemonic == "push"
            and instruction.operands[0].type == capstone.CS_OP_IMM
        ):
            extracted_values_list.append(instruction.operands[0].value.imm)

        if instruction.group(capstone.CS_GRP_JUMP) or instruction.group(
            capstone.CS_GRP_RET
        ):
            break

    if len(extracted_values_list) == 0:
        raise RuntimeError("Unable to extract the configuration")

    length_rva_encrypted = list()
    try:
        for i in range(0, len(extracted_values_list), 2):
            length_rva_encrypted.append(
                (extracted_values_list[i], extracted_values_list[i + 1])
            )
    except IndexError:
        pass
    _, rc4_va = length_rva_encrypted.pop(0)
    rc4_key = get_pe_data(pe, rc4_va, length=16)
    length, encrypted_c2_rva = length_rva_encrypted[0]
    c2_data = get_pe_data(pe, encrypted_c2_rva, length=length)
    ip_match = re.match(b"(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)", c2_data)
    doman_name_match = re.match(b"(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]*[a-zA-Z0-9])\\.)+([A-Za-z]|[A-Za-z][A-Za-z0-9\\-]*[A-Za-z0-9])", c2_data)
    
    for length, encrypted_rva in length_rva_encrypted:
        encrypted_data = get_pe_data(pe, encrypted_rva, length=length)
        if encrypted_data == b"":
            continue
        try:
            if ip_match or doman_name_match:
                decrypted_data = encrypted_data
                if b"\x00" in decrypted_data:
                    decrypted_data = decrypted_data[: decrypted_data.index(b"\x00")]
                config.append(decrypted_data.decode("utf-8"))
            else:
                decrypted_data = rc4(rc4_key, encrypted_data)
                if b"\x00" in decrypted_data:
                    decrypted_data = decrypted_data[: decrypted_data.index(b"\x00")]
                config.append(decrypted_data.decode("utf-8"))
        except UnicodeDecodeError:
            decrypted_data = rc4(rc4_key, encrypted_data)
            config.append(decrypted_data.decode("utf-8"))

    if len(config) != 0:
        netwire_config = parse_config(config)
        print(json.dumps(netwire_config, indent=4))
        return netwire_config
    else:
        raise RuntimeError("Unable to extract the configuration")
