import json
import logging
import os
import re
import sys
from optparse import OptionParser

import capstone
import pefile
import yara
from malduck import enhex, rc4, sha256

netwire_rules = [
    {
        "rule": yara.compile(
            source="rule netwire: bar {strings: $key = {C7 ?? ?? ?? 10 00 00 00 C7 ?? ?? ?? ?? ?? ?? ?? ?? ?? ?? E8 ?? ?? 00 00 ?? ?? ?? c7}  $encrypted = {C7 ?? ?? ?? ?? 00 00 00 C7 ?? ?? ?? ?? ?? ?? ?? E8 ?? ?? 00 00} condition: $key and $encrypted}"
        )
    },
    {
        "rule": yara.compile(
            source="rule netwire: bar {strings: $key = {53 55 56 57 6A 10 8D ?? ?? ?? 68 ?? ?? ?? ?? 50 E8 ?? ?? 00 00} $encrypted = {E8 ?? ?? 00 00 68 ?? 00 00 00 8D ?? ?? ?? 68 ?? ?? ?? ?? 50 e8 ?? ?? 00 00} condition: $key and $encrypted}"
        )
    },
    {
        "rule": yara.compile(
            source="rule netwire: bar {strings: $key = {C7 ?? ?? ?? 10 00 00 00 C7 ?? ?? ?? ?? ?? ?? ?? 8D ?? ?? ?? 89 ?? ?? E8 ?? ?? 00 00 C7 ?? ?? ?? FF} $encrypted = {C7 44 24 08 ?? 00 00 00 C7 44 24 04 ?? ?? ?? ?? 8D 44 24 18 89 ?? ?? E8 ?? ?? 00 00} condition: $key and $encrypted}"
        )
    },
    {
        "rule": yara.compile(
            source="rule netwire: bar {strings: $key = {8D ?? ?? ?? 89 1C 24 C7 ?? ?? ?? 10 00 00 00 C7 ?? ?? ?? ?? ?? ?? ?? E8 ?? ?? 00 00} $encrypted = {C7 ?? ?? ?? ?? 00 00 00 C7 ?? ?? ?? ?? ?? ?? ?? E8 ?? ?? 00 00} condition: $key and $encrypted}"
        )
    },
]


def print_header():
    logging.info("Author: Elastic Security (MARE)")
    logging.info(
        """
  _   _      ___          ___                             __ _                   _                  _
 | \ | |    | \ \        / (_)                           / _(_)                 | |                | |
 |  \| | ___| |\ \  /\  / / _ _ __ ___    ___ ___  _ __ | |_ _  __ _    _____  _| |_ _ __ __ _  ___| |_ ___  _ __
 | . ` |/ _ \ __\ \/  \/ / | | '__/ _ \  / __/ _ \| '_ \|  _| |/ _` |  / _ \ \/ / __| '__/ _` |/ __| __/ _ \| '__|
 | |\  |  __/ |_ \  /\  /  | | | |  __/ | (_| (_) | | | | | | | (_| | |  __/>  <| |_| | | (_| | (__| || (_) | |
 |_| \_|\___|\__| \/  \/   |_|_|  \___|  \___\___/|_| |_|_| |_|\__, |  \___/_/\_\\__|_|  \__,_|\___|\__\___/|_|
                                                                __/ |
                                                               |___/
"""
    )


def get_pe_data(pe, va, length=None):
    image_base = pe.OPTIONAL_HEADER.ImageBase
    rva = va - image_base
    data = pe.get_data(rva, length=length)
    return data


def yara_scan(data, rule_name, compiled_rule):
    matches = compiled_rule.match(data=data)
    output = matches[0].strings[0][0]
    return output


def parse_config(config):
    netwire_config = dict()
    domain_list = config[0].rstrip(";").split(";")
    netwire_config["C2 IP list"] = domain_list
    netwire_config["Password"] = config[2]
    netwire_config["Host ID"] = config[3]
    netwire_config["Mutex"] = config[4]
    netwire_config["Installation path"] = config[5]
    netwire_config["Run registry key entry"] = config[6]
    netwire_config["Active setup Key"] = config[7]
    netwire_config["Keylogger logs directory"] = config[8]
    netwire_config["Sleep in seconds"] = int(config[11])
    return netwire_config


def decrypt_memory(file, output_file=None, print_all_config=False):

    pe = pefile.PE(file)
    with open(file, "rb") as f:
        data = f.read()

    shasum: bytes = sha256(data)

    logging.info("[+] File: {}".format(file))
    config = list()
    disassembler = capstone.Cs(capstone.CS_ARCH_X86, capstone.CS_MODE_32)
    disassembler.detail = True
    for entry in netwire_rules:
        if not entry["rule"].match(data=data):
            continue
        config_decrypt_func_offset = yara_scan(data, "$key", entry["rule"])
        extracted_values_list = list()
        for _, instruction in enumerate(disassembler.disasm(data[config_decrypt_func_offset:], 0)):
            if (
                (
                    len(instruction.operands) > 1
                    and instruction.operands[1].type == capstone.CS_OP_IMM
                )
                and instruction.mnemonic == "mov"
                and instruction.operands[0].type == capstone.CS_OP_MEM
            ):
                extracted_values_list.append(instruction.operands[1].value.imm)

            if (
                instruction.mnemonic == "push"
                and instruction.operands[0].type == capstone.CS_OP_IMM
            ):
                extracted_values_list.append(instruction.operands[0].value.imm)

            if instruction.group(capstone.CS_GRP_JUMP) or instruction.group(
                capstone.CS_GRP_RET
            ):
                break

        if len(extracted_values_list) == 0:
            raise RuntimeError("Unable to extract the configuration")

        length_rva_encrypted = list()
        try:
            for i in range(0, len(extracted_values_list), 2):
                length_rva_encrypted.append(
                    (extracted_values_list[i], extracted_values_list[i + 1])
                )
        except IndexError:
            pass
        _, rc4_va = length_rva_encrypted.pop(0)
        rc4_key = get_pe_data(pe, rc4_va, length=16)
        length, encrypted_c2_rva = length_rva_encrypted[0]
        c2_data = get_pe_data(pe, encrypted_c2_rva, length=length)
        ip_match = re.match(
            b"(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)",
            c2_data,
        )
        doman_name_match = re.match(
            b"(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]*[a-zA-Z0-9])\\.)+([A-Za-z]|[A-Za-z][A-Za-z0-9\\-]*[A-Za-z0-9])",
            c2_data,
        )

        for length, encrypted_rva in length_rva_encrypted:
            encrypted_data = get_pe_data(pe, encrypted_rva, length=length)
            if encrypted_data == b"":
                continue
            try:
                if ip_match or doman_name_match:
                    decrypted_data = encrypted_data
                    if b"\x00" in decrypted_data:
                        decrypted_data = decrypted_data[: decrypted_data.index(b"\x00")]
                    config.append(decrypted_data.decode("utf-8"))
                else:
                    decrypted_data = rc4(rc4_key, encrypted_data)
                    if b"\x00" in decrypted_data:
                        decrypted_data = decrypted_data[: decrypted_data.index(b"\x00")]
                    config.append(decrypted_data.decode("utf-8"))
            except UnicodeDecodeError:
                decrypted_data = rc4(rc4_key, encrypted_data)
                config.append(decrypted_data.decode("utf-8"))
    if len(config) != 0:
        netwire_config = parse_config(config)
        netwire_config["file"] = {
            "name": file,
            "size": os.path.getsize(file),
            "hash": {"sha256": str(enhex(shasum), encoding="utf-8")},
        }

        logging.info(json.dumps(netwire_config, sort_keys=True, indent=4))
        if print_all_config:
            logging.info(
                "All the netwire configuration fields: \n{}".format(
                    ";".join([field for field in config])
                )
            )
        if output_file:
            with open(output_file, "a") as f:
                f.write(
                    json.dumps(
                        netwire_config,
                        sort_keys=True,
                    )
                )
            print("File {} was created successfully".format(output_file + ".json"))
    else:
        raise RuntimeError("Unable to extract the configuration")


def main():
    parser = OptionParser()
    parser.add_option("-f", "--file", dest="filename", help="Netwire sample path")
    parser.add_option(
        "-d", "--dir", dest="dirname", help="Directory path of Netwire samples"
    )
    parser.add_option(
        "-o", "--output-file", dest="output_file", help="Json output file"
    )
    parser.add_option(
        "-q", "--quiet", action="store_true", dest="quiet", help="quiet", default=False
    )
    parser.add_option(
        "--all-config",
        action="store_true",
        dest="print_all_config",
        help="Print the whole decrypted config",
        default=False,
    )
    (options, args) = parser.parse_args()
    file_path = options.filename
    dir_path = options.dirname
    quiet = options.quiet
    print_all_config = options.print_all_config
    output_file = options.output_file
    if quiet is True:
        log_level = logging.ERROR
    else:
        log_level = logging.INFO
    logging.basicConfig(level=log_level, format="%(message)s")

    print_header()
    if file_path is None and dir_path is None:
        parser.print_help()
        sys.exit(1)

    if file_path and os.path.isfile(file_path):
        decrypt_memory(
            file_path, output_file=output_file, print_all_config=print_all_config
        )

    if dir_path and os.path.isdir(dir_path):
        for (dirpath, _, filenames) in os.walk(dir_path):
            for file in filenames:
                try:
                    decrypt_memory(
                        os.path.join(dirpath, file),
                        output_file=output_file,
                        print_all_config=print_all_config,
                    )
                except (RuntimeError, pefile.PEFormatError):
                    print("[-] Unable to extract the configuration")
                    continue


if __name__ == "__main__":
    main()
